@@ -55,3 +55,32 @@ class NetworkChecker:
 
         # returns 1 (anomaly) when it comes to here
         exit(1)
+
+
+def dry_run_checker(args: 'argparse.Namespace'):
+    """
+    call dry run on the given endpoint
+    :param args: args provided by the CLI.
+    """
+    # No retry mechanism for dry run since it is built in the Flow
+
+    from jina import Client
+
+    client = Client(host=args.host)
+
+    try:
+
+        if client.dry_run(timeout=args.timeout):
+            default_logger.info('dry run successful')
+            exit(0)
+        else:
+            default_logger.warning('dry run failed')
+            exit(1)
+
+    except KeyboardInterrupt:
+        pass
+
+    exit(1)
+
+
+# returns 1 (anomaly) when it comes to here
