@@ -1,12 +1,14 @@
 from jina.parsers.client import mixin_comm_protocol_parser
+from jina.parsers.dryrun import set_dryrun_parser
 from jina.parsers.helper import _SHOW_ALL_ARGS
 from jina.parsers.orchestrate.runtimes.head import mixin_head_parser
 
 
-def set_pod_parser(parser=None):
+def set_pod_parser(parser=None, port_monitoring=True):
     """Set the parser for the Pod
 
     :param parser: an optional existing parser to build upon
+    :param port_monitoring: if to include the port parsing
     :return: the parser
     """
     if not parser:
@@ -31,7 +33,7 @@ def set_pod_parser(parser=None):
     mixin_container_runtime_parser(parser)
     mixin_remote_runtime_parser(parser)
     mixin_distributed_feature_parser(parser)
-    mixin_pod_parser(parser)
+    mixin_pod_parser(parser, port_monitoring=port_monitoring)
     mixin_hub_pull_options_parser(parser)
     mixin_head_parser(parser)
 
@@ -49,7 +51,7 @@ def set_deployment_parser(parser=None):
 
         parser = set_base_parser()
 
-    set_pod_parser(parser)
+    set_pod_parser(parser, port_monitoring=False)
 
     from jina.parsers.orchestrate.deployment import mixin_base_deployment_parser
 
@@ -191,6 +193,15 @@ def get_main_parser():
         )
     )
 
+    set_dryrun_parser(
+        sp.add_parser(
+            'dryrun',
+            help='Dryrun a Flow',
+            description='send a dryrun request to a Flow to check if it is working',
+            formatter_class=_chf,
+        )
+    )
+
     set_export_parser(
         sp.add_parser(
             'export',
