@@ -1,5 +1,5 @@
 import copy
-from typing import Dict, Optional, TypeVar
+from typing import Dict, Optional, TypeVar, Union
 
 from docarray import DocumentArray
 from google.protobuf import json_format
@@ -15,7 +15,13 @@ RequestSourceType = TypeVar(
 
 
 class DataRequest(Request):
-    """ Represents a DataRequest used for exchanging DocumentArrays to and within a Flow"""
+    """
+    Represents a DataRequest used for exchanging :class:`docarray.DocumentArray` with and within a Flow.
+
+    When calling :meth:`~jina.clients.mixin.PostMixin.post` on any Jina client,
+    the provided input :class:`docarray.DocumentArray` will be
+    converted to a :class:`DataRequest` before being sent to a Flow.
+    """
 
     class _DataContent:
         def __init__(self, content: 'jina_pb2.DataRequestProto.DataContentProto'):
@@ -98,6 +104,8 @@ class DataRequest(Request):
         request: Optional[RequestSourceType] = None,
     ):
         self.buffer = None
+        self._pb_body = None
+
         try:
             if isinstance(request, jina_pb2.DataRequestProto):
                 self._pb_body = request
@@ -123,28 +131,97 @@ class DataRequest(Request):
     @property
     def is_decompressed(self) -> bool:
         """
-        Checks if the underlying proto object was already deserialized
-
-        :return: True if the proto was deserialized before
+        Checks if the underlying proto object was already deserialized into a :class:`jina.proto.jina_pb2.DataRequestProto` or
+        :class:`jina.proto.jina_pb2.DataRequestProtoWoData`. This does not necessarily mean that the data (docs) inside the request is also decompressed.
+           :return: True if the proto was deserialized before
         """
-        return self.buffer is None
+        return type(self._pb_body) in [
+            jina_pb2.DataRequestProto,
+            jina_pb2.DataRequestProtoWoData,
+        ]
 
     @property
-    def proto(self) -> 'jina_pb2.DataRequestProto':
+    def is_decompressed_with_data(self) -> bool:
         """
-        Cast ``self`` to a :class:`jina_pb2.DataRequestProto`. Laziness will be broken and serialization will be recomputed when calling
+        Checks if the underlying proto object was already deserialized into a :class:`jina.proto.jina_pb2.DataRequestProto`. In this case the full proto is decompressed, including the data (docs).
+           :return: True if the proto was deserialized before, including the data (docs)
+        """
+        return type(self._pb_body) is jina_pb2.DataRequestProto
+
+    @property
+    def is_decompressed_wo_data(self) -> bool:
+        """
+        Checks if the underlying proto object was already deserialized into a :class:`jina.proto.jina_pb2.DataRequestProtoWoData`. It means that the proto is loaded without the data ( docs ).
+
+        :return: True if the proto was deserialized before into a DataRequest without docs
+        """
+        return type(self._pb_body) is jina_pb2.DataRequestProtoWoData
+
+    @property
+    def proto_wo_data(
+        self,
+    ) -> Union['jina_pb2.DataRequestProtoWoData', 'jina_pb2.DataRequestProto']:
+        """
+        Transform the current buffer to a :class:`jina_pb2.DataRequestProtoWoData` unless the full proto has already
+        been initialized or . Laziness will be broken and serialization will be recomputed when
+        calling :meth:`SerializeToString`.
+        :return: protobuf instance containing parameters
+        """
+        if self._pb_body is None:
+            self._decompress_wo_data()
+        return self._pb_body
+
+    @property
+    def proto(
+        self,
+    ) -> Union['jina_pb2.DataRequestProto', 'jina_pb2.DataRequestProtoWoData']:
+        """
+        Cast ``self`` to a :class:`jina_pb2.DataRequestProto` or a :class:`jina_pb2.DataRequestProto`. Laziness will be broken and serialization will be recomputed when calling.
+        it returns the underlying proto if it already exists (even if he is loaded without data) or creates a new one.
         :meth:`SerializeToString`.
-        :return: protobuf instance
+        :return: DataRequestProto protobuf instance
         """
         if not self.is_decompressed:
             self._decompress()
         return self._pb_body
 
-    def _decompress(self):
-        self._pb_body = jina_pb2.DataRequestProto()
+    @property
+    def proto_with_data(
+        self,
+    ) -> 'jina_pb2.DataRequestProto':
+        """
+        Cast ``self`` to a :class:`jina_pb2.DataRequestProto`. Laziness will be broken and serialization will be recomputed when calling.
+        :meth:`SerializeToString`.
+        :return: DataRequestProto protobuf instance
+        """
+        if not self.is_decompressed_with_data:
+            self._decompress()
+        return self._pb_body
+
+    def _decompress_wo_data(self):
+        """Decompress the buffer into a DataRequestProto without docs, it is useful if one want to access the parameters
+        or the header of the proto without the cost of deserializing the Docs."""
+
+        # Under the hood it used a different DataRequestProto (the DataRequestProtoWoData) that will just ignore the
+        # bytes from the bytes related to the docs that are store at the end of the Proto buffer
+        self._pb_body = jina_pb2.DataRequestProtoWoData()
         self._pb_body.ParseFromString(self.buffer)
         self.buffer = None
 
+    def _decompress(self):
+        """Decompress the buffer into a DataRequestProto"""
+        if self.buffer:
+            self._pb_body = jina_pb2.DataRequestProto()
+            self._pb_body.ParseFromString(self.buffer)
+            self.buffer = None
+        elif self.is_decompressed_wo_data:
+            self._pb_body_old = self._pb_body
+            self._pb_body = jina_pb2.DataRequestProto()
+            self._pb_body.ParseFromString(self._pb_body_old.SerializePartialToString())
+            del self._pb_body_old
+        else:
+            raise ValueError('the buffer is already decompressed')
+
     def to_dict(self) -> Dict:
         """Return the object in Python dictionary.
 
@@ -173,11 +250,11 @@ class DataRequest(Request):
 
     @cached_property
     def data(self) -> 'DataRequest._DataContent':
-        """Get the data contaned in this data request
+        """Get the data contained in this data request
 
         :return: the data content as an instance of _DataContent wrapping docs
         """
-        return DataRequest._DataContent(self.proto.data)
+        return DataRequest._DataContent(self.proto_with_data.data)
 
     @property
     def parameters(self) -> Dict:
@@ -185,15 +262,15 @@ class DataRequest(Request):
         :return: a Python dict view of the parameters.
         """
         # if u get this u need to have it decompressed
-        return json_format.MessageToDict(self.proto.parameters)
+        return json_format.MessageToDict(self.proto_wo_data.parameters)
 
     @parameters.setter
     def parameters(self, value: Dict):
         """Set the `parameters` field of this Request to a Python dict
         :param value: a Python dict
         """
-        self.proto.parameters.Clear()
-        self.proto.parameters.update(value)
+        self.proto_wo_data.parameters.Clear()
+        self.proto_wo_data.parameters.update(value)
 
     @property
     def response(self):
@@ -211,7 +288,16 @@ class DataRequest(Request):
 
         :return: the status object of this request
         """
-        return self.proto.header.status
+        return self.proto_wo_data.header.status
+
+    @property
+    def request_id(self):
+        """
+        Returns the request_id from the header field
+
+        :return: the request_id object of this request
+        """
+        return self.proto.header.request_id
 
     @classmethod
     def from_proto(cls, request: 'jina_pb2.DataRequestProto'):
@@ -222,19 +308,18 @@ class DataRequest(Request):
         return cls(request=request)
 
     def __copy__(self):
-        return DataRequest(request=self.proto)
+        return DataRequest(request=self.proto_with_data)
 
     def __deepcopy__(self, _):
-        return DataRequest(request=copy.deepcopy(self.proto))
+        return DataRequest(request=copy.deepcopy(self.proto_with_data))
 
 
 class Response(DataRequest):
     """
-    Response is the :class:`Request` object returns from the flow. Right now it shares the same representation as
-    :class:`Request`. At 0.8.12, :class:`Response` is a simple alias. But it does give a more consistent semantic on
-    the client API: send a :class:`Request` and receive a :class:`Response`.
+    Response is the :class:`~jina.types.request.Request` object returned by the flow.
 
-    .. note::
-        For now it only exposes `Docs` and `GroundTruth`. Users should very rarely access `Control` commands, so preferably
-        not confuse the user by adding `CommandMixin`.
+    At the moment it is an alias for :class:`~jina.types.request.Request`,
+    and therefore shares an identical representation.
+    Currently, its sole purpose is to give a more consistent semantic on
+    the client API: send a :class:`~jina.types.request.data.DataRequest` and receive a :class:`~jina.types.request.data.Response`.
     """
