@@ -6,6 +6,7 @@ ac_table = {
         'executor',
         'flow',
         'ping',
+        'dryrun',
         'export',
         'new',
         'gateway',
@@ -23,6 +24,7 @@ ac_table = {
             '--log-config',
             '--quiet',
             '--quiet-error',
+            '--optout-telemetry',
             '--workspace-id',
             '--extra-search-paths',
             '--timeout-ctrl',
@@ -75,12 +77,14 @@ ac_table = {
             '--log-config',
             '--quiet',
             '--quiet-error',
+            '--optout-telemetry',
             '--workspace-id',
             '--uses',
             '--env',
             '--inspect',
         ],
         'ping': ['--help', '--timeout', '--retries'],
+        'dryrun': ['--help', '--timeout'],
         'export flowchart': ['--help', '--vertical-layout'],
         'export kubernetes': ['--help', '--k8s-namespace'],
         'export docker-compose': ['--help', '--network_name'],
@@ -94,6 +98,7 @@ ac_table = {
             '--log-config',
             '--quiet',
             '--quiet-error',
+            '--optout-telemetry',
             '--workspace-id',
             '--extra-search-paths',
             '--timeout-ctrl',
@@ -163,6 +168,7 @@ ac_table = {
             '--protected-tag',
             '--force-update',
             '--force',
+            '--build-env',
             '--secret',
             '--no-cache',
             '--public',
@@ -184,6 +190,7 @@ ac_table = {
             '--log-config',
             '--quiet',
             '--quiet-error',
+            '--optout-telemetry',
             '--workspace-id',
             '--extra-search-paths',
             '--timeout-ctrl',
@@ -236,6 +243,7 @@ ac_table = {
             '--log-config',
             '--quiet',
             '--quiet-error',
+            '--optout-telemetry',
             '--workspace-id',
             '--extra-search-paths',
             '--timeout-ctrl',
